/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_SESSION_H
#define __EPP_SESSION_H

/** @file epp_Session.h
 *  @brief Definition of epp_Session and epp_Session_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "data/epp_eppdata.h"
#include "data/epp_Greeting.h"
#include "config.h"
#include "epp_Action.h"
#include "transport/transports.h"

namespace eppobject { namespace epp {
   
   /// Interface class taking care of sending command requests to the server
   /// and returning response data back. It also defines all the allowed
   /// EPP/RTK result codes.
   class epp_Session {

    private:
      epptransport::epp_TransportBase_ref m_transport; /**< The transport used to communicate with server */
      
    public:
      
      /// Null constructor, setting default transport to epptransport::epp_TransportSSL
      epp_Session()
	{
	   //Default transport is epp_TransportSSL:
	   m_transport.ref(new epptransport::epp_TransportSSL());
	};
      
      /// Destructor
      virtual ~epp_Session() {};
      
      // <FIXME>Do we want to const it?</fixme>
      /** Function that takes an epp_Action object, retrieves its request
       *  data as an #epp_string, and then calls #processXML. Then it parses the returned
       *  (response) #epp_string and puts the data back into the epp_Action object
       * @param request epp_Action request object
       * @result epp_Action result object
       * @todo This function changes the input epp_Action object, and also returns
       *       it to be consistent with the IDL. Do we want to const it?
       */
      epp_Action_ref processAction(epp_Action_ref request);

      /** Function that sends an EPP request to the server, reads the EPP response
       *  from the server, and then returns the reponse as an #epp_string
       * @param request_xml Request to send to server
       * @return Response from server
       */
      eppobject::epp::epp_string processXML(const eppobject::epp::epp_string & request_xml);
      
      /// Connects to the server, gets the epp_Greeting, and returns it
      /// @return The greeting returned from the server
      epp_Greeting_ref connectAndGetGreeting();
      
      /// Setting the transport object
      /// @param transport Transport object to use
      virtual void setTransport(const epptransport::epp_TransportBase_ref & transport)
	{
	   m_transport = transport;
	}
      
      /// Returning the active transport object
      /// @return Transport object
      virtual epptransport::epp_TransportBase_ref getTransport(void)
	{
	   return m_transport;
	}
      
      
      static const epp_short EPP_COMMAND_COMPLETED_SUCCESSFULLY = 1000;
      static const epp_short EPP_COMMAND_COMPLETED_SUCCESSFULLY_ACTION_PENDING = 1001;
      
      static const epp_short EPP_COMMAND_COMPLETED_SUCCESSFULLY_NO_MESSAGES = 1300;
      static const epp_short EPP_COMMAND_COMPLETED_SUCCESSFULLY_ACK_TO_DEQUEUE = 1301;
      /// @deprecated
      static const epp_short EPP_COMMAND_COMPLETED_SUCCESSFULLY_MESSAGE_PRESENT = 1301;
      
      static const epp_short EPP_COMMAND_COMPLETED_SUCCESSFULLY_ENDING_SESSION = 1500;
      
      static const epp_short EPP_UNKNOWN_COMMAND = 2000;
      static const epp_short EPP_COMMAND_SYNTAX_ERROR = 2001;
      static const epp_short EPP_COMMAND_USE_ERROR = 2002;
      static const epp_short EPP_REQUIRED_PARAMETER_MISSING = 2003;
      static const epp_short EPP_PARAMETER_VALUE_RANGE_ERROR = 2004;
      static const epp_short EPP_PARAMETER_VALUE_SYNTAX_ERROR = 2005;
      
      static const epp_short EPP_UNIMPLEMENTED_PROTOCOL_VERSION = 2100;
      static const epp_short EPP_UNIMPLEMENTED_COMMAND = 2101;
      static const epp_short EPP_UNIMPLEMENTED_OPTION = 2102;
      static const epp_short EPP_UNIMPLEMENTED_EXTENSION = 2103;
      static const epp_short EPP_BILLING_FAILURE = 2104;
      static const epp_short EPP_OBJECT_IS_NOT_ELIGIBLE_FOR_RENEWAL = 2105;
      static const epp_short EPP_OBJECT_IS_NOT_ELIGIBLE_FOR_TRANSFER = 2106;
      
      static const epp_short EPP_AUTHENTICATION_FAILURE = 2200;
      static const epp_short EPP_AUTHORIZATION_FAILURE = 2201;
      static const epp_short EPP_INVALID_AUTHORIZATION_IDENTIFIER = 2202;
      static const epp_short EPP_OBJECT_PENDING_TRANSFER = 2300;
      static const epp_short EPP_OBJECT_NOT_PENDING_TRANSFER = 2301;
      static const epp_short EPP_OBJECT_EXISTS = 2302;
      static const epp_short EPP_OBJECT_DOES_NOT_EXIST = 2303;
      static const epp_short EPP_OBJECT_STATUS_PROHIBITS_OPERATION = 2304;
      static const epp_short EPP_OBJECT_ASSOCIATION_PROHIBITS_OPERATION = 2305;
      static const epp_short EPP_PARAMETER_VALUE_POLICY_ERROR = 2306;
      static const epp_short EPP_UNIMPLEMENTED_OBJECT_SERVICE = 2307;
      static const epp_short EPP_DATA_MANAGEMENT_POLICY_VIOLATION = 2308;
      static const epp_short EPP_COMMAND_FAILED = 2400;
      static const epp_short EPP_COMMAND_FAILED_SERVER_CLOSING_CONNECTION = 2500;
      /// @deprecated
      static const epp_short EPP_COMMAND_FAILED_SERVER_ENDING_SESSION = 2500;
      static const epp_short EPP_AUTHENTICATION_ERROR_SERVER_CLOSING_CONNECTION = 2501;
      /// @deprecated
      static const epp_short EPP_TIMEOUT_SERVER_ENDING_SESSION = 2501;
      static const epp_short EPP_SESSION_LIMIT_EXCEEDED_SERVER_CLOSING_CONNECTION = 2502;
      static const epp_short RTK_COMMUNICATIONS_FAILURE = 2600;
      static const epp_short RTK_UNEXPECTED_SERVER_DISCONNECT = 2601;
      
   };

   /// Typedef for the ref to the epp_Session class
   typedef refcnt_ptr<epp_Session> epp_Session_ref;
   
}}

#endif
