/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_TRANSPORTBASE_H
#define __EPP_TRANSPORTBASE_H

/** @file epp_TransportBase.h
 *  @brief Definition of epp_TransportBase and epp_TrException
 *  @author Asbjorn Steira Mikkelsen
 */

#include <domtools/refcount.h>
#include <string>
#include <exception>
#include <cstdio>

namespace epptransport {

   /// Abstract class defining common methods and attributes used in all transport classes
   class epp_TransportBase {
      
    protected:
      std::string m_type; /**< Transport type */
      
    public:
      
      /// Null constructor
      epp_TransportBase()
	{
	   m_type="epp_TransportBase";
	};
      
      /// Destructor 
      virtual ~epp_TransportBase() {};
      
      /// Abstract function, writing data to server
      /// @param xml_string Data to write to server
      virtual void writeToServer(const std::string & xml_string) = 0;
      
      /// Abstract function, reading data from server
      /// @return Data read from server
      virtual std::string readFromServer() = 0;
      
      /// Connects to server
      virtual void connect() {};
      
      /// Disconnects from server
      virtual void disconnect() {};
      
      /// Returning the port of the server
      /// @return Server port
      virtual unsigned long getServerPort() { return 0;};
      
      /// Setting the port of the server
      /// @param serverPort Server port
      virtual void setServerPort(const unsigned long &) {};
      
      /// Returning the name of the server
      /// @return Server name
      virtual std::string getServerName() { return "";};
      
      /// Setting the name of the server
      /// @param serverName Server name
      virtual void setServerName(const std::string &) {};
      
      /// Returning type of transport
      /// @return Transport type
      virtual std::string getType() const
	{
	   return m_type;
	}
   };
   
   /// Typedef for the ref to the epp_TransportBase class
   typedef refcnt_ptr<epp_TransportBase> epp_TransportBase_ref;
   
   /// Exception class containing information about errors encountered in transport handling
   class epp_TrException : public std::exception {
      
    public:
      std::string m_error_message; /**< Text describing the encountered error */
      
      /// Null constructor
      epp_TrException() {};
      
      /// Destructor
      virtual ~epp_TrException() throw() {};
      
      /// Constructor with error data
      epp_TrException(const std::string & data)
	{
	   m_error_message = data;
	};
      
      /// Returns the error string
      /// @deprecated Access the data directly instead
      virtual std::string getString() const
	{
	   return m_error_message;
	}
      
      /// Constructor with error data, file name, and line number
      epp_TrException(const std::string & file, const int lineno, const std::string & data)
	{
	   char buf[10];
	   sprintf(buf,"%d",lineno);
	   m_error_message = file;
	   m_error_message += ":";
	   m_error_message += buf;
	   m_error_message += ": ";
	   m_error_message = data;
	};      
   };
   
}

#endif
