/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_TRANSPORTTCP_H
#define __EPP_TRANSPORTTCP_H

/** @file epp_TransportTCP.h
 *  @brief Definition of epp_TransportTCP
 *  @author Asbjorn Steira Mikkelsen
 */

#include <string>
#include "epp_TransportConn.h"


namespace epptransport {

   /// Class defining the TCP transport, which reads from and
   /// writes to a socket
   /// @see epp_TransportConn for additional information
   class epp_TransportTCP : public epp_TransportConn {
      
    private:
      
      char readbuf[16384]; /**< Buffer used when reading from server */
      int sockfd; /**< Socket file descriptor */
      unsigned int pos; /**< Read position in read buffer */
      int len; /**< Length of data in read buffer */
      unsigned int timeout; /**< Timeout value */
      
      /// Reads data from the socket into #readbuf
      /// @return True if successful read from socket
      bool fillBuf(void);
      
      /// Read or write operation
      enum {
	 READ      = 1,
	 WRITE     = 2
      };
      
    public:
      
      /// Null constructor, setting epp_TransportBase#m_type
      epp_TransportTCP();

      /// Destructor
      virtual ~epp_TransportTCP()
	{
	   // To avoid the destructor from throwing an exception:
	   try {
	      disconnect();
	   } catch(...) {
	   }
	};

      /// Writing data to server
      /// @param xml_string Data to write to server
      /// @throw epp_TrException If it cannot write to server
      void writeToServer(const std::string & xml_string);
      
      /// Reading data from server
      /// @return Data read from server
      std::string readFromServer();

      /// Selects type of operation (READ/WRITE)
      /// @param type Type of operation to select
      /// @return True if operation successfully selected
      /// @throw epp_TrException If socket timed out
      bool select(int type) const;
      
      /// Connecting to server by setting server name and server port and then calling #connect()
      /// @param serverName Name of server
      /// @param serverPort Port of server
      void connect(const std::string & serverName, const unsigned long & serverPort);
      
      /// Connects to server
      /// @throw epp_TrException If it is unable to resolve hostname
      /// @throw epp_TrException If it is unable to connect to server
      void connect();
      
      /// Disconnects from server
      /// @note <fixme>Make content of function private, and make this one virtual, and
      ///       make all usage of function call the private function instead.</fixme>
      void disconnect();
      
      /// Reads one character from the server
      /// @return ASCII value of read character, or -1 if nothing to read
      int getC();
   };
   
   /// Typedef for the ref to the epp_TransportTCP class
   typedef refcnt_ptr<epp_TransportTCP> epp_TransportTCP_ref;
   
}

#endif
